;Colour bars program
;-------------------
;Binary created using DAsm 2.12 running on an Amiga.

   PROCESSOR   6502

BALLPHIY    EQU   #$0000
BALLPLOY    EQU   #$0700
BALLPHIX    EQU   #$0002
BALLPLOX    EQU   #$0703
BALLVLOY    EQU   #$0004
BALLVHIY    EQU   #$0005
BALLVLOX    EQU   #$0006
BALLVHIX    EQU   #$0007
BALLACCY    EQU   #$0008
BALLACCX    EQU   #$0009
BALLVISIBLE EQU   #$000A
FIRED       EQU   #$000B
HIT         EQU   #$000C
MISSED      EQU   #$000D
READY       EQU   #$000E
CONTROLLER1 EQU   #$0010
CONTROLLER2 EQU   #$0011

;----------------------

BALLCHR     EQU   #$0701
P1VHIY      EQU   #$0711         ;Sprites for Y,X Initial Velocity
P1VLOY      EQU   #$0715
P1VHIX      EQU   #$0719
P1VLOX      EQU   #$071D

   ORG   $C000    ;16Kb PRG-ROM, 8Kb CHR-ROM

Reset_Routine  SUBROUTINE
   cld         ;Clear decimal flag
   sei         ;Disable interrupts
.WaitV   lda $2002
   bpl .WaitV     ;Wait for vertical blanking interval
   ldx #$00
   stx $2000
   stx $2001      ;Screen display off, amongst other things
   dex
   txs         ;Top of stack at $1FF

;Clear (most of) the NES' WRAM. This routine is ripped from "Duck Hunt" - I should probably clear all
;$800 bytes.
   ldy #$07    ;To clear 8 x $100 bytes, from $000 to $7FF?
   sty $01        ;Store count value in $01
   ldy #$00
   sty $00
   lda #$00

.Clear   sta ($00),y    ;Clear $100 bytes
   dey
   bne .Clear

   dec $01        ;Decrement "banks" left counter
   bpl .Clear     ;Do next if >= 0


   lda   #$20
   sta   $2006
   lda   #$00
   sta   $2006

   ldx   #$00
   ldy   #$10
.ClearPPU sta $2007
   dex
   bne   .ClearPPU
   dey
   bne   .ClearPPU

   lda   #$00
   sta   $2003
   tay
.ClearSpr sta   $2004
   dey
   bne   .ClearSpr

;********* Initialize Palette to specified colour ********

   ldx   #$3F
   stx   $2006
   ldx   #$00
   stx   $2006

   ldx   #$0D     ;Colour Value (Black)
   ldy   #$20     ;Clear BG & Sprite palettes.
.InitPal stx $2007
   dey
   bne   .InitPal
;*********************************************************

   lda   #$3F
   sta   $2006
   lda   #$01
   sta   $2006
   lda   #$30
   sta   $2007
   sta   $2007
   sta   $2007

   lda   #$3F
   sta   $2006
   lda   #$11
   sta   $2006
   lda   #$30
   sta   $2007
   sta   $2007
   sta   $2007

   lda   #$21
   sta   $2006
   lda   #$10
   sta   $2006
   lda   #$41
   sta   $2007

   lda   #$25
   sta   $2006
   lda   #$10
   sta   $2006
   lda   #$42
   sta   $2007

;-----------------BALL DATA ----------------------------

   lda   #$00
   sta   FIRED
   sta   HIT
   sta   MISSED

   lda   #$01
   sta   READY

   lda   #$00
   sta   BALLPHIY
   lda   #$80
   sta   BALLPLOY

   lda   #$00
   sta   BALLPHIX
   lda   #$20
   sta   BALLPLOX

   lda   #$01
   sta   BALLVHIY
   lda   #$00
   sta   BALLVLOY

   lda   #$00
   sta   BALLVHIX
   lda   #$40
   sta   BALLVLOX

   lda   #$05
   sta   BALLACCY
   lda   #$00
   sta   BALLACCX

   lda   #$01
   sta   BALLVISIBLE


;   lda   #$00           ;Set up sprite palette & position
;   sta   $2003
;   lda   BALLPLOY
;   sta   $2004
;   lda   BALLVISIBLE
;   sta   $2004
;   lda   #$00
;   sta   $2004
;   lda   BALLPLOX
;   sta   $2004
;------------------------------
   lda   #$20       ;Set up Velocity Sprites
   sta   $0710
   sta   $0714
   lda   #$28
   sta   $0718
   sta   $071C

   lda   #$10
   sta   $0713
   sta   $071B
   lda   #$18
   sta   $0717
   sta   $071F
;-------------------------------

;Enable vblank interrupts, etc.
   lda   #%10001000
   sta   $2000
   lda   #%00011110        ;Screen on, sprites on, show leftmost 8 pixels, colour
   sta   $2001
;  cli            ;Enable interrupts(?)

;Now just loop forever?
.Loop

   lda   READY
   bne   .ReadyForControllerRead
   jmp   .NotReady

.ReadyForControllerRead
   lda   CONTROLLER1
   and   #%00001000
   beq   .NotUp
   lda   CONTROLLER1
   and   #%11110111
   sta   CONTROLLER1
   lda   BALLVLOY
   clc
   adc   #$01
   sta   BALLVLOY
   lda   BALLVHIY
   adc   #$00
   sta   BALLVHIY

   lda   BALLVLOY
   sta   P1VLOY
   lda   BALLVHIY
   ora   #$10
   sta   P1VHIY
   lda   P1VLOY
   lsr
   lsr
   lsr
   lsr
   ora   #$10
   sta   P1VLOY

.NotUp

   lda   CONTROLLER1
   and   #%00000100
   beq   .NotDown
   lda   CONTROLLER1
   and   #%11111011
   sta   CONTROLLER1
   lda   BALLVLOY
   sec
   sbc   #$01
   sta   BALLVLOY
   lda   BALLVHIY
   sbc   #$00
   sta   BALLVHIY

   lda   BALLVLOY
   sta   P1VLOY
   lda   BALLVHIY
   ora   #$10
   sta   P1VHIY
   lda   P1VLOY
   lsr
   lsr
   lsr
   lsr
   ora   #$10
   sta   P1VLOY


.NotDown

   lda   CONTROLLER1
   and   #%00000001
   beq   .NotRight
   lda   CONTROLLER1
   and   #%11111110
   sta   CONTROLLER1
   lda   BALLVLOX
   clc
   adc   #$01
   sta   BALLVLOX
   lda   BALLVHIX
   adc   #$00
   sta   BALLVHIX

   lda   BALLVLOX
   sta   P1VLOX
   lda   BALLVHIX
   ora   #$10
   sta   P1VHIX
   lda   P1VLOX
   lsr
   lsr
   lsr
   lsr
   ora   #$10
   sta   P1VLOX

.NotRight

   lda   CONTROLLER1
   and   #%00000010
   beq   .NotLeft
   lda   CONTROLLER1
   and   #%11111101
   sta   CONTROLLER1
   lda   BALLVLOX
   sec
   sbc   #$01
   sta   BALLVLOX
   lda   BALLVHIX
   sbc   #$00
   sta   BALLVHIX

   lda   BALLVLOX
   sta   P1VLOX
   lda   BALLVHIX
   ora   #$10
   sta   P1VHIX
   lda   P1VLOX
   lsr
   lsr
   lsr
   lsr
   ora   #$10
   sta   P1VLOX

.NotLeft
   lda   CONTROLLER1
   and   #%10000000          ;A
   beq   .NotA

   lda   #$01
   sta   FIRED
   lda   #$00
   sta   MISSED
   lda   #$00
   sta   READY
   lda   #$01
   sta   BALLVISIBLE


.NotA

.NotReady

   jmp   .Loop

Gravity SUBROUTINE
   lda   BALLVLOY
   sec
   sbc   BALLACCY
   sta   BALLVLOY
   bcs   .NoOverflowVY
   dec   BALLVHIY
.NoOverflowVY
   lda   BALLVHIY
   pha
   lda   BALLVLOY
   pha
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY
   lsr   BALLVHIY
   ror   BALLVLOY

   lda   BALLPLOY
   sec
   sbc   BALLVLOY
   sta   BALLPLOY

   pla
   sta   BALLVLOY
   pla
   sta   BALLVHIY

;---------------------- X-POS -------------
   lda   BALLVLOX
   clc
   adc   BALLACCX
   sta   BALLVLOX
   bcc   .NoOverflowX
   lda   BALLVHIX
   sec
   adc   #$00
   sta   BALLVHIX
.NoOverflowX
   lda   BALLVHIX
   pha
   lda   BALLVLOX
   pha
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX
   lsr   BALLVHIX
   ror   BALLVLOX

   lda   BALLPLOX
   clc
   adc   BALLVLOX
   sta   BALLPLOX
   lda   #$00
   adc   BALLPHIX
   sta   BALLPHIX

   pla
   sta   BALLVLOX
   pla
   sta   BALLVHIX

;------------ TESTING FOR OFF-SCREEN ------------------------------
   lda   BALLPHIY
   beq   .Test1
   dec   BALLPHIY
   beq   .Test2
   dec   BALLPHIY
   beq   .Test3

.Test1 lda   #$10                    ;Disappear if goes too high
   sec
   sbc   BALLPLOY
   bcc   .GravEnd
                                     ;HOW DOES THIS WORK????????
   lda   #$01
   sta   BALLVISIBLE
   lda   #$01
   sta   BALLPHIY
   jmp   .GravEnd

.Test2
   lda   #$01
   sta   BALLPHIY
   lda   #$F0
   sec
   sbc   BALLPLOY
   bcs   .GravEnd

   lda   #$00
   sta   BALLVISIBLE
   lda   #$02
   sta   BALLPHIY
   jmp   .GravEnd

.Test3
   lda   #$02
   sta   BALLPHIY
   lda   #$40
   sec
   sbc   BALLPLOY
   bcs   .GravEnd

   lda   #$00
   sta   BALLPHIY

.GravEnd

;------------ TEST FOR MISS -------------
   lda   FIRED
   beq   .NoHitDetect
   lda   BALLVISIBLE
   beq   .NoHitDetect
   lda   #$D0
   cmp   BALLPLOY
   bcs   .NotYMissed

   lda   #$01
   sta   MISSED
   lda   #$01
   sta   READY
   lda   #$00
   sta   FIRED
   lda   #$00
   sta   BALLVISIBLE

   sta   BALLPLOX
   sta   BALLPHIX
   sta   BALLPLOY
   sta   BALLPHIY
.NotYMissed
   lda   BALLPHIX
   cmp   #$02
   bmi   .NotXMissed
   lda   #$01
   sta   MISSED
   lda   #$01
   sta   READY
   lda   #$00
   sta   FIRED
   lda   #$00
   sta   BALLVISIBLE
   lda   #$00
   sta   BALLPLOX
   sta   BALLPHIX
   sta   BALLPLOY
   sta   BALLPHIY
.NotXMissed
.NoHitDetect
;------------------------------------
   rts

NMI_Routine SUBROUTINE
   php
   pha
   txa
   pha
   tya
   pha

;---------- READ JOYPAD -----------------------

   ldx   #$01
   stx   $4016
   dex
   stx   $4016
   ldy   #$08     ;Read 8 bits from $4016
.ReadCont1
   lda   $4016
   ror
   txa
   rol
   tax
   dey
   bne   .ReadCont1
   sta   CONTROLLER1
;---------------------------------------------
   lda   FIRED
   beq   .NotFired
   lda   HIT
   bne   .Hit
   lda   MISSED
   bne   .Missed
   jsr   Gravity

.NotFired
.Hit
.Missed

;   lda   #$00
;   sta   $2003
;   lda   BALLPLOY
;   sta   $2004
;   stx   $2004
;   lda   #$00
;   sta   $2004
;   lda   BALLPLOX
;   sta   $2004

   ldx   BALLVISIBLE
   inx
   stx   BALLCHR

   lda   #$07
   sta   $4014             ;Copy $700 to Spr-RAM

;----------- SET SCROLLING -------------
   lda   BALLPHIX
   beq   .ScrollScreenA
   lda   #$FF
   sta   $2005
   jmp   .SetVScroll
.ScrollScreenA lda   #$00
   sta   $2005
.SetVScroll lda   #$00
   sta   $2005
;---------------------------------------

;   lda   BALLPHIX
;   lsr
;   lda   BALLPLOX
;   ror
;   sta   $2005
;   lda   #$00
;   sta   $2005

   pla
   tay
   pla
   tax
   pla
   plp
   rti

IRQ_Routine       ;Dummy label
   rti

;That's all the code. Now we just need to set the vector table approriately.

   ORG   $FFFA,0
   dc.w  NMI_Routine
   dc.w  Reset_Routine
   dc.w  IRQ_Routine    ;Not used, just points to RTI


;The end.
